Attribute VB_Name = "modEvaluation"
'Evaluation Module
'
'Contains all functions and subs for evaluation of a string, including a true/false
'analyzer.  For RPN, see modRPN.

Option Explicit

Public Function GetValue(ByRef Start As Long, ByVal Parameter As String, Optional Eval As Boolean = True, Optional ByVal InWhat As String) As String
    'This will search for the paramter, starting at the specified location, skipping
    'things in quotes, then evalutate it.
    Dim FinalCode As String, WhereItIs As Long
    
    If InWhat = "" Then InWhat = dScript
    WhereItIs = CodeInStr(Start, InWhat, Parameter)
    
    If WhereItIs = 0 Then
        FlagError "Syntax Error", "Expected '" & Parameter & "'", iLocation: Exit Function
    Else
        FinalCode = Mid(InWhat, Start, WhereItIs - Start)
        
        GetValue = Trim(FinalCode)
        If Eval Then GetValue = Solve(GetValue)
        
        Start = Start + Len(FinalCode) + Len(Parameter)
    End If
End Function

Public Function GetValue3(ByRef Start As Long, ByVal InWhat As String) As String
    'Similar to GetValue, however accounts for parenthesis
    Dim FinalCode As String
    Dim MakeInt As Integer, sChar As String
    Dim Depth As Integer, Quote As String
    Dim x As Integer
    
    If InWhat = "" Then InWhat = dScript
    
    For x = Start To Len(InWhat)
        sChar = Mid(InWhat, x, 1)
        If sChar = ")" Then
            If Depth = 0 Then
                FinalCode = Mid(InWhat, Start, x - Start)
                GetValue3 = Trim(FinalCode)
                Start = Start + Len(FinalCode) + 1
                Exit Function
            Else
                Depth = Depth - 1
            End If
        ElseIf sChar = Chr(34) Then
            Quote = InStr(x + 1, InWhat, Chr(34))
            If Quote <> 0 Then
                x = Quote
            End If
        ElseIf sChar = "(" Then
            Depth = Depth + 1
        End If
    Next 'x
    
    FlagError "Syntax Error", "Expected ')'", iLocation: Exit Function
End Function

Public Function GetFunctions(ByVal sFunction As String, Optional flagTypeError As Boolean = True) As String
    'On Error GoTo error
    Dim tArgs() As String, Args() As typeVar
    Dim ProcFunction As String, ProcArguments As String
    Dim TempFunction As String, tempInt As Integer
    Dim n As Integer, Quote As Long
    Dim i As Long, j As Integer
    Dim tempString As String
    
    sFunction = Trim(sFunction)
    i = InStr(1, sFunction, "(")
    ProcFunction = Trim(LCase(Mid(sFunction, 1, i - 1)))
    ProcArguments = Trim(GetValue3(i + 1, Trim(sFunction)))

    If ProcArguments <> "" Then 'solve out all given arguments
        tArgs = dSplit(ProcArguments, ",")    'split arguments
        ReDim Args(UBound(tArgs))
        For n = LBound(Args) To UBound(Args) 'solve all arguments
            Args(n) = makeIntoVar(tArgs(n))
        Next 'n
    End If
        
    Select Case ProcFunction
        Case "lcase"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = LCase(Args(0).Value)
            End If
        Case "ucase"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = UCase(Args(0).Value)
            End If
        Case "trim"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = Trim(Args(0).Value)
            End If
        Case "isnumeric"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = IsNumeric(Args(0).Value)
            End If
        Case "isop"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = IsOperator(Args(0).Value)
            End If
        Case "int"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = Int(Args(0).Value)
            End If
        Case "fileexist"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = FileExist(Args(0).Value)
            End If
        Case "rndnum"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                Randomize
                TempFunction = Int(Rnd * Int(Args(0).Value))
            End If
        Case "rnd"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                Randomize
                TempFunction = (Rnd * Args(0).Value)
            End If
        Case "asc"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = Asc(Args(0).Value)
            End If
        Case "chr"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = Chr(Args(0).Value)
            End If
        Case "hex"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = Hex(Args(0).Value)
            End If
        Case "len"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = Len(Args(0).Value)
            End If
        Case "val"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = Val(Args(0).Value)
            End If
        Case "string"
            If UBound(Args) <> 1 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = String(Args(0).Value, Args(1).Value)
            End If
        Case "right"
            If UBound(Args) <> 1 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = Right(Args(0).Value, Args(1).Value)
            End If
        Case "left"
            If UBound(Args) <> 1 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = Left(Args(0).Value, Args(1).Value)
            End If
        Case "instr"
            If UBound(Args) <> 2 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = InStr(Args(0).Value, Args(1).Value, Args(2).Value)
            End If
        Case "mid"
            If UBound(Args) = 1 Then
                TempFunction = Mid(Args(0).Value, Args(1).Value)
            ElseIf UBound(Args) = 2 Then
                TempFunction = Mid(Args(0).Value, Args(1).Value, Args(2).Value)
            Else
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            End If
        'FILE FUNCTIONS
        Case "readfile"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = ReadFile(Args(0).Value)
            End If
        Case "f_open"
            If UBound(Args) <> 1 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = doFileOpen(Args(0).Value, Args(1).Value)
            End If
        Case "f_out"
            If UBound(Args) <> 1 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                Print #Args(0).Value, Args(1).Value
            End If
        Case "f_close"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                Close #Args(0).Value
            End If
        Case "f_linein"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                Line Input #Args(0).Value, tempString
                TempFunction = tempString
            End If
        Case "f_in"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                Input #Args(0).Value, Args(1).Value
                TempFunction = Args(1).Value
            End If
        Case "eof"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = EOF(Args(0).Value) 'note: accepts file number
            End If
        Case "lof"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = LOF(Args(0).Value) 'note: accepts file number
            End If
        Case "filelen"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                TempFunction = FileLen(Args(0).Value) 'note: accepts full file name
            End If
        'ARRAY FUNCTIONS
        Case "isarray"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                tempInt = GetVarIndex(Args(0).Name, True)
                
                TempFunction = dVars(tempInt).isArray
            End If
        Case "isempty"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                tempInt = GetVarIndex(Args(0).Name, True)
                
                If dVars(tempInt).isArray Then
                    TempFunction = IIf(dVars(tempInt).numElements = -1, True, False)
                Else
                    TempFunction = IIf(dVars(tempInt).Value = "", True, False)
                End If
            End If
        Case "ubound"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                tempInt = GetVarIndex(Args(0).Name, True)
                If Not dVars(tempInt).isArray Then FlagError "Run-Time Error", "Variable type mismatch: " & vbCrLf & dVars(tempInt).Name & ": darray <-> dvar", iLocation
                
                TempFunction = dVars(tempInt).numElements
            End If
        Case "split"
            FlagError "Run-Time Error", "Variable type mismatch: " & vbCrLf & dVars(tempInt).Name & ": darray <-> dvar" & vbCrLf & "(cannot call Split() from here)", iLocation
        
        'CONSOLE FUNCTIONS
        Case "console.length", "conlen"
            If ProcArguments <> "" Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                If appDOS Then Exit Function
                TempFunction = Len(frmRun.txtText.Text)
            End If
        Case "console.setstart"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                If appDOS Then Exit Function
                frmRun.txtText.SelStart = Val(Args(0).Value)
            End If
        Case "console.settext"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                If appDOS Then Exit Function
                frmRun.txtText.Text = Args(0).Value
            End If
        Case "console.gettext"
            If ProcArguments <> "" Then
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                If appDOS Then Exit Function
                TempFunction = frmRun.txtText.Text
            End If
        Case "console.lock"
            If UBound(Args) <> 0 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                If appDOS Then Exit Function
                Select Case Args(0).Value
                    Case 0
                        frmRun.txtText.Locked = False
                    Case 1
                        frmRun.txtText.Locked = True
                End Select
            End If
        Case Else 'user defined function

            If LCase(ProcFunction) = "main" Then FlagError "Syntax Error", "Cannot call 'main'", iLocation: Exit Function
            
            If Not hasFunctions Then _
                FlagError "Syntax Error", "Function undefined: " & ProcFunction, iLocation: Exit Function
            
            tempInt = GetProcIndex(ProcFunction, True)
            
            Dim NewCode As String
            Dim OldCode As String, OldProc As String
            Dim Args2 As Variant
            Dim returnVarIndex As Integer
            
            NewCode = dProcs(tempInt).Code  'get the function's code from code array
            
            Args2 = dSplit(dProcs(tempInt).Parameters, ",") 'split declaration arguments
            
            'make it so we can return values
            ReturnCount = ReturnCount + 1
            ReDim Preserve ProcReturns(ReturnCount)
            
            If dProcs(tempInt).isArray And flagTypeError Then _
                FlagError "Run-Time Error", "Variable type mismatch: " & vbCrLf & "dvar <-> darray", iLocation

            
            OldCode = dScript           'save the old code
            OldProc = CurrentProc       'save the old proc
            CurrentProc = ProcFunction  'set the current proc
            
            If Trim(ProcArguments) <> "" And Trim(dProcs(tempInt).Parameters) <> "" Then
                'check for correct number of args by comparing declaration and provided vars
                If UBound(Args) <> UBound(Args2) Then
                    FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
                End If
    
                'create vars for this function
                For n = LBound(Args) To UBound(Args)
                    AddVar Trim(Args2(n)), Args(n).Value, ProcFunction, False
                    If Args(n).isArray Then
                        'FlagError "Syntax Error", "Can't predefine number of elements when declaring argument", iLocation: Exit Function
                        dVars(VarCount).arrayValues = Args(n).arrayValues
                        dVars(VarCount).numElements = Args(n).numElements
                        dVars(VarCount).isArray = True
                    End If
                Next 'n
            End If
            
            'parse all new code, based on this function
            frmRun.LinkCode NewCode, False
            ClearVars ProcFunction  'clear all vars for this function
            CurrentProc = OldProc   'reset to old proc
            dScript = OldCode       'reset to old code

            'add the return value
            TempFunction = ProcReturns(ReturnCount).Value
             
            ProcReturns(ReturnCount).Value = ""   'reset returns
            ReturnCount = ReturnCount - 1   'ditto
            
    End Select
    
    GetFunctions = TempFunction
    Exit Function
    
error:
    FlagError "Run-Time Error", "Error: " & err.Description, iLocation
End Function

Public Function GetArrayFunctions(ByVal sFunction As String) As String()
    'almost exactly the same as the above function, except it returns an array
    'used ONLY for an expression, where the left side is an array
    'On Error GoTo error
    Dim tArgs() As String, Args() As typeVar
    Dim ProcFunction As String, ProcArguments As String
    Dim TempFunction As String, tempInt As Integer
    Dim n As Integer, Quote As Long
    Dim i As Long, j As Integer
    
    sFunction = Trim(sFunction)
    i = InStr(1, sFunction, "(")
    ProcFunction = Trim(LCase(Mid(sFunction, 1, i - 1)))
    ProcArguments = Trim(GetValue3(i + 1, Trim(sFunction)))
        
    If Trim(ProcArguments) <> "" Then
        'solve out all given arguments
        tArgs = dSplit(ProcArguments, ",")    'split arguments
        ReDim Args(UBound(tArgs))
        For n = LBound(Args) To UBound(Args) 'solve all arguments
            Args(n) = makeIntoVar(tArgs(n))
        Next 'n
    End If
        
    Select Case ProcFunction
        Case "split"
            If UBound(Args) <> 1 Then 'Check for correct arguments
                FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
            Else
                GetArrayFunctions = Split(Args(0).Value, Args(1).Value)
            End If
        Case Else 'user defined function
           If LCase(ProcFunction) = "main" Then FlagError "Syntax Error", "Cannot call 'main'", iLocation: Exit Function
            
           If Not hasFunctions Then _
            FlagError "Syntax Error", "Function undefined: " & ProcFunction, iLocation: Exit Function
            
            tempInt = GetProcIndex(ProcFunction, True)
            
            Dim NewCode As String
            Dim OldCode As String, OldProc As String
            Dim Args2 As Variant
            Dim returnVarIndex As Integer
            
            NewCode = dProcs(tempInt).Code  'get the function's code from code array
            Args2 = dSplit(dProcs(tempInt).Parameters, ",") 'split declaration arguments
            
            'make it so we can return values
            ReturnCount = ReturnCount + 1
            ReDim Preserve ProcReturns(ReturnCount)
            
            If Not dProcs(tempInt).isArray Then _
                FlagError "Run-Time Error", "Type mismatch: " & vbCrLf & "darray <-> dvar", iLocation
            
            ProcReturns(ReturnCount).isArray = True
            
            OldCode = dScript           'save the old code
            OldProc = CurrentProc       'save the old proc
            CurrentProc = ProcFunction  'set the current proc
            
            If Trim(ProcArguments) <> "" And Trim(dProcs(tempInt).Parameters) <> "" Then
                'check for correct number of args by comparing declaration and provided vars
                If UBound(Args) <> UBound(Args2) Then
                    FlagError "Syntax Error", "Wrong number of arguments for " & ProcFunction & "().", iLocation: Exit Function
                End If
    
                'create vars for this function
                For n = LBound(Args) To UBound(Args)
                    AddVar Trim(Args2(n)), Args(n).Value, ProcFunction, False
                    If Args(n).isArray Then
                        dVars(VarCount).arrayValues = Args(n).arrayValues
                        dVars(VarCount).numElements = Args(n).numElements
                        dVars(VarCount).isArray = True
                    End If
                Next 'n
            End If
            
            'parse all new code, based on this function
            frmRun.LinkCode NewCode, False
            ClearVars ProcFunction  'clear all vars for this function
            CurrentProc = OldProc   'reset to old proc
            dScript = OldCode       'reset to old code
            
            'add the return value
            GetArrayFunctions = ProcReturns(ReturnCount).arrayValues
            
            Erase ProcReturns(ReturnCount).arrayValues    'reset returns
            ReturnCount = ReturnCount - 1   'ditto

    End Select

Exit Function
error:
    FlagError "Run-Time Error", "Error: " & err.Description, iLocation
End Function


Public Function Solve(ByVal sFunction As String) As String
    'this is the master solver
    
    If Trim(sFunction) = "" Then Exit Function  'makes sure it's something
    
    'sFunction = GetFunctions(sFunction)         'first solve all functions
    Solve = RPNcalculate(sFunction)             'send it to the RPN engine
End Function

Public Function IsOperator(ByVal NumVal As String) As Boolean
    'Determines if a character is an operator
    Select Case NumVal
    Case "+", "-", "*", "/", "\", "&", ">", "<", "=", "^", "%"
        IsOperator = True
    End Select
End Function

Public Function DoOperation(ByVal LeftVal As String, ByVal Operator As String, ByVal RightVal As String) As String
    'Solves an equation
    Select Case Operator
        Case "+"
            DoOperation = Val(LeftVal) + Val(RightVal)
        Case "-"
            DoOperation = Val(LeftVal) - Val(RightVal)
        Case "/"
            If Val(RightVal) = 0 Then FlagError "Syntax Error", "Division by zero.", iLocation: Exit Function
            DoOperation = Val(LeftVal) / Val(RightVal)
        Case "*"
            DoOperation = Val(LeftVal) * Val(RightVal)
        Case "^"
            DoOperation = Val(LeftVal) ^ Val(RightVal)
        Case "%"
            DoOperation = Val(LeftVal) Mod Val(RightVal)
        Case "&"
            DoOperation = LeftVal & RightVal
        Case "\"
            DoOperation = Val(LeftVal) \ Val(RightVal)
        Case Else
            If Operator = "" Then
                FlagError "Syntax Error", "Operator required for expression.", iLocation: Exit Function
            Else
                FlagError "Syntax Error", "Invalid operator in OP1, '" & Operator & "'", iLocation: Exit Function
            End If
    End Select
End Function

Public Function doEval(ByVal LeftVal As String, ByVal Operator As String, ByVal RightVal As String) As Boolean
    'Determines if an expression is True or False
    
    Select Case Operator
        Case ">"
            If IsNumeric(LeftVal) Or IsNumeric(RightVal) Then
                If Val(LeftVal) > Val(RightVal) Then doEval = True
            Else
                If LeftVal > RightVal Then doEval = True
            End If
        Case "<"
            If IsNumeric(LeftVal) Or IsNumeric(RightVal) Then
                If Val(LeftVal) < Val(RightVal) Then doEval = True
            Else
                If LeftVal < RightVal Then doEval = True
            End If
        Case "="
            If IsNumeric(LeftVal) Or IsNumeric(RightVal) Then
                If Val(LeftVal) = Val(RightVal) Then doEval = True
            Else
                If LeftVal = RightVal Then doEval = True
            End If
        Case "<>", "!="
            If IsNumeric(LeftVal) Or IsNumeric(RightVal) Then
                If Val(LeftVal) <> Val(RightVal) Then doEval = True
            Else
                If LeftVal <> RightVal Then doEval = True
            End If
        Case ">="
            If IsNumeric(LeftVal) Or IsNumeric(RightVal) Then
                If Val(LeftVal) >= Val(RightVal) Then doEval = True
            Else
                If LeftVal >= RightVal Then doEval = True
            End If
        Case "<="
            If IsNumeric(LeftVal) Or IsNumeric(RightVal) Then
                If Val(LeftVal) <= Val(RightVal) Then doEval = True
            Else
                If LeftVal <= RightVal Then doEval = True
            End If
        Case Else
            If Operator = "" Then
                If RightVal = "" Then   'this makes it so you can do statements like "if a then", no = operator needed
                    If IsNumeric(LeftVal) Then
                        If Val(LeftVal) <> 0 Then doEval = True
                    Else
                        If LeftVal = True Then doEval = True
                    End If
                Else
                    FlagError "Syntax Error", "Operator required for comparison.", iLocation: Exit Function
                End If
            Else
                FlagError "Syntax Error", "Invalid operator in Eval, '" & Operator & "'", iLocation: Exit Function
            End If
    End Select
End Function

Public Function Eval(ByVal sFunction As String) As Boolean
    'This parses a string into a left value, operator, and right value and determines
    'if the statement is true or false
    
    Dim LeftVal As String, RightVal As String, Operator
    Dim sChar As String, OpFound As Boolean
    Dim x As Integer
    
    For x = 1 To Len(sFunction)
        sChar = Mid(sFunction, x, 1)
        If sChar = ">" Or sChar = "<" Or sChar = "=" Or sChar = "!" And Not OpFound Then
            Operator = Operator & sChar
            OpFound = True
        Else
            If OpFound Then
                RightVal = RightVal & sChar
            Else
                LeftVal = LeftVal & sChar
            End If
        End If
    Next x
    
    LeftVal = Solve(LeftVal)
    RightVal = Solve(RightVal)
    
    Eval = doEval(LeftVal, Operator, RightVal)
End Function

Public Function FindIdentifier(StartPos As Long, InWhat As String) As String
    On Error Resume Next
    Dim i As Long
    
    For i = StartPos To 1 Step -1
        Select Case Mid(InWhat, i - 1, 1)
            Case " ", ";", vbTab, "("
                If i > StartPos Then GoTo SkipCurrentI
                FindIdentifier = Trim(Mid(InWhat, i, StartPos - (i - 1)))
                If FindIdentifier <> "" Then Exit Function
        End Select
        If i = 2 Then Exit For
        
        If Mid(InWhat, i - 2, 2) = vbCrLf Then
            If i > StartPos Then GoTo SkipCurrentI
            FindIdentifier = Trim(Mid(InWhat, i, StartPos - (i - 1)))
            If FindIdentifier <> "" Then Exit Function
            Exit Function
        End If
SkipCurrentI:
    Next i
    FindIdentifier = Trim(Mid(InWhat, 1, StartPos))
End Function

Public Function isMathThingy(Thingy As String) As Boolean
    isMathThingy = False
    
    If IsOperator(Thingy) Then isMathThingy = True
    If IsNumeric(Thingy) Then isMathThingy = True

End Function
